@@ -2,7 +2,7 @@
 This file is part of the `arabluatex' package
 
 ArabLuaTeX -- Processing ArabTeX notation under LuaLaTeX
-Copyright (C) 2016--2017  Robert Alessi
+Copyright (C) 2016--2018  Robert Alessi
 
 Please send error reports and suggestions for improvements to Robert
 Alessi <alessi@robertalessi.net>
@@ -31,17 +31,18 @@ punctuationtr = {
 
 nulltr = {
    {a="%|", b=""},
-   {a="o[%S]-o", b=""}
+   {a="o[%S]-o", b=""},
+   {a="[%^%_](.)", b="<??>%1"}
 }
 
--- cap
+-- cap (legacy)
 captr = {
    -- dmg (defaut); loc as well
    {a="ā", b="Ā"},
    {a="b", b="B"},
    {a="t", b="T"},
    {a="ṯ", b="Ṯ"},
-   {a="ğ", b="Ğ"},
+   {a="ǧ", b="Ǧ"},
    {a="ḥ", b="Ḥ"},
    {a="ḫ", b="Ḫ"},
    {a="d", b="D"},
@@ -68,9 +69,65 @@ captr = {
    {a="ī", b="Ī"}
 }
 
+-- uc
+lcuc = {
+   {a="b", b="B"},
+   {a="t", b="T"},
+   {a="ṯ", b="Ṯ"},
+   {a="ǧ", b="Ǧ"},
+   {a="j", b="J"},
+   {a="ḥ", b="Ḥ"},
+   {a="ḫ", b="Ḫ"},
+   {a="d", b="D"},
+   {a="ḏ", b="Ḏ"},
+   {a="r", b="R"},
+   {a="z", b="Z"},
+   {a="s", b="S"},
+   {a="š", b="Š"},
+   {a="ṣ", b="Ṣ"},
+   {a="ḍ", b="Ḍ"},
+   {a="ṭ", b="Ṭ"},
+   {a="ẓ", b="Ẓ"},
+   {a="ġ", b="Ġ"},
+   {a="f", b="F"},
+   {a="q", b="Q"},
+   {a="k", b="K"},
+   {a="l", b="L"},
+   {a="m", b="M"},
+   {a="n", b="N"},
+   {a="h", b="H"},
+   {a="w", b="W"},
+   {a="y", b="Y"},
+   {a="u", b="U"},
+   {a="a", b="A"},
+   {a="i", b="I"},
+   {a="ū", b="Ū"},
+   {a="ā", b="Ā"},
+   {a="ī", b="Ī"},
+   -- additional characters
+   {a="p", b="P"},
+   {a="č", b="Č"},
+   {a="ž", b="Ž"},
+   {a="v", b="V"},
+   {a="g", b="G"},
+   {a="ñ", b="Ñ"},
+   {a="ch", b="Ch"}, -- loc
+}
+
 -- dmg
 
 hamzatrdmg = {
+   -- next lines for ʾalif alone
+   {a="(%.A)l%-(%^n)", b=".|l-%2"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)l%-(%^n)", b="%1.|l-%3"}, --additional (^n is lunar) --p
+   {a="(%.A)l%-([%_%^%.]?[tdrzsnc])", b=".|%2-%2"},
+   {a="([%(%[%|%<%s%-o])(%.A)l%-([%_%^%.]?[tdrzsnc])", b="%1.|%3-%3"}, --p
+   {a="(%.A)([uai])l%-(%^n)", b="||%2l-%3"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)([uai])l%-(%^n)", b="%1||%3l-%4"}, --additional (^n is lunar) --p
+   {a="(%.A)([uai])l%-([%_%^%.]?[tdrzsnc])", b="||%2%3-%3"},
+   {a="([%(%[%|%<%s%-o])(%.A)([uai])l%-([%_%^%.]?[tdrzsnc])", b="%1||%3%4-%4"}, --p
+   {a="(%.A)([^uai])", b=".|%2"},
+   {a="(%.A)([uai])", b="||%2"},
    -- hard coded hamza
    {a="|\"'", b="ʾ"},
    {a="A\"'", b="ʾA"},
@@ -88,6 +145,10 @@ hamzatrdmg = {
    -- taḫfīfu 'l-hamza
    {a="'u'([^uaiUAI])", b="ʾU%1"},
    {a="'i'([^uaiUAI])", b="ʾI%1"},
+   {a="^u'([^uaiUAI])", b="U%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1U%2"},
+   {a="^i'([^uaiUAI])", b="I%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1I%2"},
    -- madda (historic writing below)
    {a="'a'([^uaiUAI])", b="ʾA%1"},
    {a="'a?A", b="ʾA"},
@@ -108,8 +169,11 @@ hamzatrdmg = {
    -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="ʾ%2"},
    {a="^(')(i)", b="ʾ%2"},
-   {a="(%W)(')([ua])", b="%1ʾ%3"},
-   {a="(%W)(')(i)", b="%1ʾ%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1ʾ%3"},
+   --   {a="(%W)(')(i)", b="%1ʾ%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1ʾ%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1ʾ%3"},
    -- final
    {a="([Iy])(')(aN)$", b="%1ʾ%3"},
    {a="([Iy])(')(aN)(%W)", b="%1ʾ%3%4"},
@@ -121,15 +185,15 @@ hamzatrdmg = {
    {a="(U)(')", b="%1ʾ"},
    {a="([Iy])(')", b="%1ʾ"},
    {a="([^uai])(')([uU])", b="%1ʾ%3"},
-   {a="([^uai])(')([aA])", b="%1ʾ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1ʾ%3"},
    {a="([^uai])(')([iI])", b="%1ʾ%3"},
    {a="(u)(')([uU])", b="%1ʾ%3"},
-   {a="(u)(')([aA])", b="%1ʾ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(u)(')([iI])", b="%1ʾ%3"},
-   {a="(a)(')([aA])", b="%1ʾ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(a)(')([uU])", b="%1ʾ%3"},
    {a="(a)(')([iI])", b="%1ʾ%3"},
-   {a="(i)(')([aA])", b="%1ʾ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(i)(')([uU])", b="%1ʾ%3"},
    {a="(i)(')([iI])", b="%1ʾ%3"},
    {a="(a)(')([^uaiUAI])", b="%1ʾ%3"},
@@ -140,6 +204,8 @@ hamzatrdmg = {
 tanwintrdmg = {
    {a="%-?([uai]NU)(o)([ui])([%S]-o)", b="\\arbup{un%3}"},
    {a="%-?([uai]NU)(%s)([ui])", b="\\arbup{un%3}%2'"},
+   {a="%-?(iNI)(o)([ui])([%S]-o)", b="i\\arbup{n%3}"},
+   {a="%-?(iNI)(%s)([ui])", b="i\\arbup{n%3}%2'"},
    {a="(o[%S]-)([uai]N[UI])(o)(\"?[ui])", b="'"},
    {a="%-?uNU", b="\\arbup{un}"},
    {a="%-?aNU", b="\\arbup{an}"},
@@ -211,12 +277,16 @@ trigraphstrdmg = { -- trigraphs or more
    -- al- + lām
    {a="^(a)l%-(l)", b="%1l-%2"},
    {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1%2l-%3"}, --p
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="%1%2-%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsn])", b="%1%2%3-%3"}, --p
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="%1%2-"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsn])%-", b="%1%2%3-"}, --p
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="%1l-%2"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1%2l-%3"}, --^n is lunar --p
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2-%2"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2%3-%3"}, --p
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="%1l-"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1%2l-"}, --^n is lunar --p
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2-"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2%3-"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="%1l-%2"},
    {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1%2l-%3"}, --p
@@ -233,15 +303,23 @@ trigraphstrdmg = { -- trigraphs or more
    -- art. with waṣla + lām
    {a="'l%-(l)", b="'l-%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="'%1-%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="'l-%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="'%1-%1"},
    -- li-/la- + art. + lām
-   {a="l([ai])%-l%-(l)", b="l%1-%2%2"},
+   {a="l([ai])%-l%-(l)", b="l%1-%2-%2"},
    -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="'%1-"},
+   -- ('c' and '^n' are additional characters)
+   {a="'(%^n)%-", b="'l-"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="'%1-"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="l%1-%2-%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="l%1-l-%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="l%1-%2-%2"},
    -- li-/la- + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="l%1-%2-%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="l%1-l-%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="l%1-%2-%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-(\"?[uai])", b="'l-%1"},
    -- art. with waṣla + lunar consonant (i.e. what remains)
@@ -299,16 +377,16 @@ digraphstrdmg = {
    {a="([%_]?[Uu])(%s)([%(%[%|%<]?)(\"?[uai])", b="u%2%3'"}, --p
    {a="([%_]?[Ii])(%s)([%(%[%|%<]?)(\"?[uai])", b="i%2%3'"}, --p
    -- ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[uai])(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[uai])(%p?)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?ayni)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[uai])(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[uai])(%p*)$", b="\\arbup{%2}%3"},
    -- ʾiʿrāb hyphen (end) shorten long vowels preceding ʾalif
    -- conjunctionis—without forgetting 'lla_dI
    {a="(U)(A)", b="U"},
@@ -328,20 +406,20 @@ digraphstrdmg = {
    {a="([%_]?[Uu])(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="u%2%3"},
    {a="([%_]?[Ii])(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="i%2%3"},
    {a="%-%-", b=""},
-   {a="iyyaT(%p?)$", b="īyaT%1"},
-   {a="iyyaT(%p?%s)", b="īyaT%1"},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyyaT(%p*)$", b="īyaT%1"},
+   {a="iyyaT(%p*%s)", b="īyaT%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    --   {a="T([^uai])", b="%1"},
    {a="T(\\arbup)", b="t%1"},
    {a="([a%'][%_%^%.]?[tdrzsln]%-)(%S-)T([%(%[%|%<%s])(a[%_%^%.]?[tdrzsln]%-)", b="%1%2h%3%4"}, --p
    {a="T([%(%[%|%<%s])(a[%_%^%.]?[tdrzsln]%-)", b="t%1%2"}, --p
    {a="T([%|\"])", b="t%1"},
-   {a="T(%p?%s)", b="h%1"},
-   {a="T(%p?)$", b="h%1"},
-   {a="T(%p?)(%W)", b="h%1%2"},
+   {a="T(%p*%s)", b="h%1"},
+   {a="T(%p*)$", b="h%1"},
+   {a="T(%p*)(%W)", b="h%1%2"},
    {a="_t", b="ṯ"},
-   {a="%^g", b="ğ"},
+   {a="%^g", b="ǧ"},
    {a="%.h", b="ḥ"},
    {a="_h", b="ḫ"},
    {a="_d", b="ḏ"},
@@ -351,6 +429,12 @@ digraphstrdmg = {
    {a="%.t", b="ṭ"},
    {a="%.z", b="ẓ"},
    {a="%.g", b="ġ"},
+   {a="%.y", b="y"},
+   -- additional characters (begin)
+   {a="%^c", b="č"},
+   {a="%^z", b="ž"},
+   {a="%^n", b="ñ"},
+   -- additional characters (end)
    -- the following needs to be moved above shortening rules
 --   {a="(U)(A)", b="ū"},
    {a="WA", b="w"},
@@ -370,7 +454,7 @@ digraphstrdmg = {
 singletrdmg = {
    {a="b", b="b"},
    {a="t", b="t"},
-   {a="j", b="ğ"},
+   {a="j", b="ǧ"},
    {a="x", b="ḫ"},
    {a="d", b="d"},
    {a="r", b="r"},
@@ -387,6 +471,11 @@ singletrdmg = {
    {a="w", b="w"},
    {a="y", b="y"},
    {a="T", b="t"},
+   -- additional characters (begin)
+   {a="p", b="p"},
+   {a="v", b="v"},
+   {a="g", b="g"},
+   -- additional characters (end)
    {a="\"", b=""},
    {a="B", b=""}
 }
@@ -395,10 +484,14 @@ longvtrdmg = {
    {a="A", b="ā"},
    {a="U", b="ū"},
    {a="I", b="ī"},
+   {a="aY", b="ay"},
+   {a="iY", b="ī"},
    {a="Y", b="ạ̄"}
 }
 
 shortvtrdmg = {
+   {a="([uai])([uai])([uai])", b="/%1,%2,%3/"},
+   {a="([uai])([uai])", b="/%1,%2/"},
    {a="u", b="u"},
    {a="a", b="a"},
    {a="i", b="i"}
@@ -407,6 +500,9 @@ shortvtrdmg = {
 -- loc
 
 hamzatrloc = {
+   -- next lines for ʾalif alone
+   {a="(%.A)([^uai])", b=".|%2"},
+   {a="(%.A)([uai])", b="||%2"},
    -- hard coded hamza
    {a="|\"'", b="ʾ"},
    {a="A\"'", b="ʾA"},
@@ -423,11 +519,13 @@ hamzatrloc = {
    {a="%'%_I", b="I"},
    -- taḫfīfu 'l-hamza
    {a="^'u'([^uaiUAI])", b="U%1"},
-   {a="(%W)'u'([^uaiUAI])", b="%1U%2"},
-   {a="'u'([^uaiUAI])", b="ʾU"},
+   {a="([%s%(%[%<%-])'u'([^uaiUAI])", b="%1U%2"},
    {a="^'i'([^uaiUAI])", b="I%1"},
-   {a="(%W)'i'([^uaiUAI])", b="%1I%2"},
-   {a="'i'([^uaiUAI])", b="ʾI"},
+   {a="([%s%(%[%<%-])'i'([^uaiUAI])", b="%1I%2"},
+   {a="^u'([^uaiUAI])", b="U%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1U%2"},
+   {a="^i'([^uaiUAI])", b="I%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1I%2"},
    -- madda (historic writing below)
    {a="^(')(A)", b="%2"},
    {a="(%W)(')(A)", b="%1%3"},   
@@ -444,8 +542,11 @@ hamzatrloc = {
    -- initial (needs both ^ and %W patterns)
    {a="^(')([ua])", b="%2"},
    {a="^(')(i)", b="%2"},
-   {a="(%W)(')([ua])", b="%1%3"},
-   {a="(%W)(')(i)", b="%1%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1%3"},
+   --   {a="(%W)(')(i)", b="%1%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1%3"},
    -- final
    {a="([Iy])(')(aN)$", b="%1ʾ%3"},
    {a="([Iy])(')(aN)(%W)", b="%1ʾ%3%4"},
@@ -457,15 +558,15 @@ hamzatrloc = {
    {a="(U)(')", b="%1ʾ"},
    {a="([Iy])(')", b="%1ʾ"},
    {a="([^uai])(')([uU])", b="%1ʾ%3"},
-   {a="([^uai])(')([aA])", b="%1ʾ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1ʾ%3"},
    {a="([^uai])(')([iI])", b="%1ʾ%3"},
    {a="(u)(')([uU])", b="%1ʾ%3"},
-   {a="(u)(')([aA])", b="%1ʾ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(u)(')([iI])", b="%1ʾ%3"},
-   {a="(a)(')([aA])", b="%1ʾ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(a)(')([uU])", b="%1ʾ%3"},
    {a="(a)(')([iI])", b="%1ʾ%3"},
-   {a="(i)(')([aA])", b="%1ʾ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(i)(')([uU])", b="%1ʾ%3"},
    {a="(i)(')([iI])", b="%1ʾ%3"},
    {a="(a)(')([^uaiUAI])", b="%1ʾ%3"},
@@ -477,6 +578,7 @@ tanwintrloc = {
    {a="%-?uNU", b="un"},
    {a="%-?aNU", b="an"},
    {a="%-?iNU", b="in"},
+   {a="%-?iNI", b="in"},
    {a="%-?(\"?At)%-?([ui])N", b="%1%2n"},
    {a="%-?([ui])N", b="%1n"},
    {a="%-?(aN)(_A)", b="an"},
@@ -492,12 +594,16 @@ trigraphstrloc = { -- trigraphs or more
    -- al- + lām
    {a="^(a)l%-(l)", b="%1l-%2"},
    {a="(%s)(a)l%-(l)", b="%1%2l-%3"},
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="%1l-%2"},
-   {a="(%s)(a)l%-([%_%^%.]?[tdrzsn])", b="%1%2l-%3"},
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="%1l-"},
-   {a="(%s)(a)([%_%^%.]?[tdrzsn])%-", b="%1%2l-"},
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="%1l-%2"}, -- ^n is lunar
+   {a="(%s)(a)l%-(%^n)", b="%1%2l-%3"}, -- ^n is lunar
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="%1l-%2"},
+   {a="(%s)(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2l-%3"},
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="%1l-"}, -- ^n is lunar
+   {a="(%s)(a)(%^n)%-", b="%1%2l-"}, -- ^n is lunar
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="%1l-"},
+   {a="(%s)(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2l-"},
    -- al- + initial unstable hamza
    {a="^(a)l%-([uai])", b="%1l-%2"},
    {a="(%s)(a)l%-([uai])", b="%1%2l-%3"},
@@ -509,15 +615,23 @@ trigraphstrloc = { -- trigraphs or more
    -- art. with waṣla + lām
    {a="'l%-(l)", b="al-%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="al-%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="al-%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="al-%1"},
    -- li-/la- + art. + lām
    {a="l([ai])%-l%-(l)", b="l%1-l-%2"},
    -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="al-"},
+   -- ('c' and '^n' are additional characters)
+   {a="'(%^n)%-", b="al-"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="al-"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="l%1-l-%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="l%1-l-%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="l%1-l-%2"},
    -- li-/la- + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="l%1-l-%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="l%1-l-%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="l%1-l-%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-([uai])", b="al-%1"},   
    -- art. with waṣla + lunar consonant (i.e. what remains)
@@ -537,31 +651,32 @@ trigraphstrloc = { -- trigraphs or more
 
 digraphstrloc = {
    -- discard the ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
-   {a="(%-)([uai])(%p?%s)", b="%2%3"},
-   {a="(%-)([uai])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="%2%3"},
+   {a="(%-)([uai])(%p*%s)", b="%2%3"},
+   {a="(%-)([uai])(%p*)$", b="%2%3"},
    -- discard the ʾiʿrāb hyphen (end)
    {a="(%-)(\"?[uai])", b="%1%2"}, -- hyphen + initial alif without hamza
    {a="^(\"?[uai])", b="%1"},      -- initial alif without hamza
    {a="(%s)([uai])", b="%1%2"}, -- initial alif without hamza
    {a="%-%-", b=""},
    {a="uww", b="ūw"},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    {a="iyy", b="īy"},
    {a="([tkdsg])(h)", b="%1'%2"},
    --   {a="T([^uai])", b="h%1"},
    {a="([a%']l%-)(%S-)T([%(%[%|%<%s])(al%-)", b="%1%2h%3%4"}, --p
    {a="T([%(%[%|%<%s])(al%-)", b="t%1%2"}, --p
-   {a="T(%p?)$", b="h%1"},
-   {a="T(%p?%s)", b="h%1"},
+   {a="T([%|\"])", b="t%1"},
+   {a="T(%p*)$", b="h%1"},
+   {a="T(%p*%s)", b="h%1"},
    {a="_t", b="th"},
    {a="%^g", b="j"},
    {a="%.h", b="ḥ"},
@@ -573,6 +688,12 @@ digraphstrloc = {
    {a="%.t", b="ṭ"},
    {a="%.z", b="ẓ"},
    {a="%.g", b="gh"},
+   {a="%.y", b="y"},
+   -- additional characters (begin)
+   {a="%^c", b="ch"},
+   {a="%^z", b="zh"},
+   {a="%^n", b="ñ"},
+   -- additional characters (end)
    {a="(U)(A)", b="ū"},
    {a="WA", b="w"},
    {a="(a)W", b="%1w"},
@@ -608,6 +729,11 @@ singletrloc = {
    {a="w", b="w"},
    {a="y", b="y"},
    {a="T", b="t"},
+   -- additional characters (begin)
+   {a="p", b="p"},
+   {a="v", b="v"},
+   {a="g", b="g"},
+   -- additional characters (end)
    {a="\"", b=""},
    {a="B", b=""}
 }
@@ -616,10 +742,14 @@ longvtrloc = {
    {a="A", b="ā"},
    {a="U", b="ū"},
    {a="I", b="ī"},
+   {a="aY", b="ay"},
+   {a="iY", b="ī"},
    {a="Y", b="á"},
 }
 
 shortvtrloc = {
+   {a="([uai])([uai])([uai])", b="/%1,%2,%3/"},
+   {a="([uai])([uai])", b="/%1,%2/"},
    {a="u", b="u"},
    {a="a", b="a"},
    {a="i", b="i"}
@@ -632,6 +762,9 @@ finaltrloc = {
 -- arabica
 
 hamzatrarabica = { -- ≠ from hamzatrloc: initial hamza has to be held
+   -- next lines for ʾalif alone
+   {a="(%.A)([^uai])", b=".|%2"},
+   {a="(%.A)([uai])", b="||%2"},
    -- hard coded hamza
    {a="|\"'", b="ʾ"},
    {a="A\"'", b="ʾA"},
@@ -648,11 +781,13 @@ hamzatrarabica = { -- ≠ from hamzatrloc: initial hamza has to be held
    {a="%'%_I", b="I"},
    -- taḫfīfu 'l-hamza
    {a="^'u'([^uaiUAI])", b="U%1"},
-   {a="(%W)'u'([^uaiUAI])", b="%1U%2"},
-   {a="'u'([^uaiUAI])", b="ʾU"},
+   {a="([%s%(%[%<%-])'u'([^uaiUAI])", b="%1U%2"},
    {a="^'i'([^uaiUAI])", b="I%1"},
-   {a="(%W)'i'([^uaiUAI])", b="%1I%2"},
-   {a="'i'([^uaiUAI])", b="ʾI"},
+   {a="([%s%(%[%<%-])'i'([^uaiUAI])", b="%1I%2"},
+   {a="^u'([^uaiUAI])", b="U%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])u'([^uaiUAI])", b="%1U%2"},
+   {a="^i'([^uaiUAI])", b="I%1"},
+   {a="([^uaiUAIYN][%s%(%[%<])i'([^uaiUAI])", b="%1I%2"},
    -- madda (historic writing below)
    {a="^(')(A)", b="%2"},
    {a="(%W)(')(A)", b="%1%3"},
@@ -670,8 +805,11 @@ hamzatrarabica = { -- ≠ from hamzatrloc: initial hamza has to be held
    -- hold it for now (see below, beginning of digraphs table)
    {a="^(')([ua])", b="@%2"},
    {a="^(')(i)", b="@%2"},
-   {a="(%W)(')([ua])", b="%1@%3"},
-   {a="(%W)(')(i)", b="%1@%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1@%3"},
+   --   {a="(%W)(')(i)", b="%1@%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1@%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1@%3"},
    -- final
    {a="([Iy])(')(aN)$", b="%1ʾ%3"},
    {a="([Iy])(')(aN)(%W)", b="%1ʾ%3%4"},
@@ -683,15 +821,15 @@ hamzatrarabica = { -- ≠ from hamzatrloc: initial hamza has to be held
    {a="(U)(')", b="%1ʾ"},
    {a="([Iy])(')", b="%1ʾ"},
    {a="([^uai])(')([uU])", b="%1ʾ%3"},
-   {a="([^uai])(')([aA])", b="%1ʾ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1ʾ%3"},
    {a="([^uai])(')([iI])", b="%1ʾ%3"},
    {a="(u)(')([uU])", b="%1ʾ%3"},
-   {a="(u)(')([aA])", b="%1ʾ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(u)(')([iI])", b="%1ʾ%3"},
-   {a="(a)(')([aA])", b="%1ʾ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(a)(')([uU])", b="%1ʾ%3"},
    {a="(a)(')([iI])", b="%1ʾ%3"},
-   {a="(i)(')([aA])", b="%1ʾ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ʾ%3"},
    {a="(i)(')([uU])", b="%1ʾ%3"},
    {a="(i)(')([iI])", b="%1ʾ%3"},
    {a="(a)(')([^uaiUAI])", b="%1ʾ%3"},
@@ -706,12 +844,16 @@ trigraphstrarabica = { -- trigraphs or more
    -- al- + lām
    {a="^(a)l%-(l)", b="%1l-%2"},
    {a="(%s)(a)l%-(l)", b="%1%2l-%3"},
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="%1l-%2"},
-   {a="(%s)(a)l%-([%_%^%.]?[tdrzsn])", b="%1%2l-%3"},
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="%1l-"},
-   {a="(%s)(a)([%_%^%.]?[tdrzsn])%-", b="%1%2l-"},
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="%1l-%2"}, -- ^n is lunar
+   {a="(%s)(a)l%-(%^n)", b="%1%2l-%3"}, -- ^n is lunar
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="%1l-%2"},
+   {a="(%s)(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2l-%3"},
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="%1l-"}, -- ^n is lunar
+   {a="(%s)(a)(%^n)%-", b="%1%2l-"}, -- ^n is lunar
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="%1l-"},
+   {a="(%s)(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2l-"},
    -- al- + initial unstable hamza
    {a="^(a)l%-([uai])", b="%1l-%2"},
    {a="(%s)(a)l%-([uai])", b="%1%2l-%3"},
@@ -723,15 +865,23 @@ trigraphstrarabica = { -- trigraphs or more
    -- art. with waṣla + lām
    {a="'l%-(l)", b="l-%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="l-%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="l-%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="l-%1"},
    -- li-/la- + art. + lām
    {a="l([ai])%-l%-(l)", b="l%1-l-%2"},
    -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="l-"},
+   -- ('c' and '^n' are additional characters)
+   {a="'(%^n)%-", b="l-"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="l-"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="l%1-l-%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="l%1-l-%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="l%1-l-%2"},
    -- li-/la- + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="l%1-l-%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="l%1-l-%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="l%1-l-%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-([uai])", b="l-%1"},   
    -- art. with waṣla + lunar consonant (i.e. what remains)
@@ -751,32 +901,35 @@ trigraphstrarabica = { -- trigraphs or more
 
 digraphstrarabica = {
    {a="([uai]%-)(\"?[uai])", b="%1"}, -- hyphen + initial alif without hamza
+   {a="([UAIYuai])(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2%3"}, --p
+   {a="(o[%S]-)([UAIuai])(o)(\"?[uai])", b=""},
    {a="@", b=""}, -- remove the tag before the former hamza
    -- discard the ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
-   {a="(%-)([uai])(%p?%s)", b="%2%3"},
-   {a="(%-)([uai])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="%2%3"},
+   {a="(%-)([uai])(%p*%s)", b="%2%3"},
+   {a="(%-)([uai])(%p*)$", b="%2%3"},
    -- discard the ʾiʿrāb hyphen (end)
    {a="(%-)(\"?[uai])", b="%1%2"}, -- hyphen + initial alif without hamza
    {a="^(\"?[uai])", b="%1"},      -- initial alif without hamza
    {a="(%s)([uai])", b="%1%2"}, -- initial alif without hamza
    {a="%-%-", b=""},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    --   {a="T([^uai])", b="h%1"},
    {a="([a%']l%-)(%S-)aT([%(%[%|%<%s])(al%-)", b="%1%2a%3%4"}, --p
    {a="aT([%(%[%|%<%s])(al%-)", b="at%1%2"}, --p
-   {a="aT(%p?)$", b="a%1"},
-   {a="aT(%p?%s)", b="a%1"},
+   {a="T([%|\"])", b="t%1"},
+   {a="aT(%p*)$", b="a%1"},
+   {a="aT(%p*%s)", b="a%1"},
    {a="_t", b="ṯ"},
-   {a="%^g", b="ğ"},
+   {a="%^g", b="ǧ"},
    {a="%.h", b="ḥ"},
    {a="_h", b="ḫ"},
    {a="_d", b="ḏ"},
@@ -786,6 +939,12 @@ digraphstrarabica = {
    {a="%.t", b="ṭ"},
    {a="%.z", b="ẓ"},
    {a="%.g", b="ġ"},
+   {a="%.y", b="y"},
+   -- additional characters (begin)
+   {a="%^c", b="č"},
+   {a="%^z", b="ž"},
+   {a="%^n", b="ñ"},
+   -- additional characters (end)
    {a="(U)(A)", b="ū"},
    {a="WA", b="w"},
    {a="(a)W", b="%1w"},
@@ -804,7 +963,7 @@ digraphstrarabica = {
 singletrarabica = {
    {a="b", b="b"},
    {a="t", b="t"},
-   {a="j", b="ğ"},
+   {a="j", b="ǧ"},
    {a="x", b="ḫ"},
    {a="d", b="d"},
    {a="r", b="r"},
@@ -821,11 +980,18 @@ singletrarabica = {
    {a="w", b="w"},
    {a="y", b="y"},
    {a="T", b="t"},
+   -- additional characters (begin)
+   {a="p", b="p"},
+   {a="v", b="v"},
+   {a="g", b="g"},
+   -- additional characters (end)
    {a="\"", b=""},
    {a="B", b=""}
 }
 
 longvtrarabica = {
+   {a="aY", b="ay"},
+   {a="iY", b="ī"},
    {a="[AY]", b="ā"},
    {a="U", b="ū"},
    {a="I", b="ī"}
