@@ -2,7 +2,7 @@
 This file is part of the `arabluatex' package
 
 ArabLuaTeX -- Processing ArabTeX notation under LuaLaTeX
-Copyright (C) 2016--2017  Robert Alessi
+Copyright (C) 2016--2018  Robert Alessi
 
 Please send error reports and suggestions for improvements to Robert
 Alessi <alessi@robertalessi.net>
@@ -22,6 +22,13 @@ along with this program.  If not, see
 <http://www.gnu.org/licenses/>.
 --]]
 
+arbmarks = {
+   {a="bismillah", b="^^^^fdfd", c="\\arb[trans]{bi-ismi \\uc{'l-l_ahi} 'l-ra.hm_ani 'l-ra.hImi}"},
+   {a="salam", b="^^^^fdf5", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"},
+   {a="slm", b="^^^^fdfa", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"},
+   {a="jalla", b="^^^^fdfb", c="\\arb[trans]{^galla ^galAla-hu}"}
+}
+
 abjad = {
 {"a\"'", "b", "j", "d", "h", "w", "z", ".h", ".t"},
 {"y", "k", "l", "m", "n", "s", "`", "f", ".s", },
@@ -205,10 +212,10 @@ hamza = {
    {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
-   {a="(A)(')(uN?%p?)$", b="aآء%3"},
-   {a="(A)(')(uN?)(%p?%s)", b="aآء%3%4"},
-   {a="(A)(')(iN?%p?)$", b="aآء%3"},
-   {a="(A)(')(iN?)(%p?%s)", b="aآء%3%4"},
+   {a="(A)(')(uN?%p*)$", b="aآء%3"},
+   {a="(A)(')(uN?)(%p*%s)", b="aآء%3%4"},
+   {a="(A)(')(iN?%p*)$", b="aآء%3"},
+   {a="(A)(')(iN?)(%p*%s)", b="aآء%3%4"},
    {a="(A)(')([iI])", b="aآئ%3"}, -- historic madda
    {a="(A)(')(u)", b="aآؤ%3"}, -- historic madda
    {a="(A)(')", b="aآء"}, -- historic madda
@@ -238,31 +245,31 @@ hamza = {
    {a="(mi)(%')(aN%_?[AY])", b="%1أ%3"},
    -- final hamzah is on the line after a letter of prolongation or a
    -- consonant with sukūn
-   {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
-   {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
+   {a="([^Auai])(')(\"?[uai]N?)(%p*)$", b="%1ء%3%4"}, --new
+   {a="([^Auai])(')(\"?[uai]N?)(%p*%s)", b="%1ء%3%4"},
 -- u
-   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
-   {a="(u)(')(%p?)$", b="%1ؤ%3"},
-   {a="(u)(')(%p?%s)", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?%p*)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p*%s)", b="%1ؤ%3%4"},
+   {a="(u)(')(%p*)$", b="%1ؤ%3"},
+   {a="(u)(')(%p*%s)", b="%1ؤ%3"},
 -- a
-   {a="(a)(')(A%p?)$", b="%1آ"},
-   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
-   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
-   {a="(a)(')(a%p?)$", b="%1أ%3"},
-   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
-   {a="(a)(')(aN%p?)$", b="%1أً"},
-   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
-   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
-   {a="(a)(')(%p?)$", b="%1أ%3"},
-   {a="(a)(')(%p?%s)", b="%1أ%3"},
+   {a="(a)(')(A%p*)$", b="%1آ"},
+   {a="(a)(')(A)(%p*%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p*)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p*%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p*)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p*%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p*)$", b="%1أً"},
+   {a="(a)(')(aN)(%p*%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p*)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p*%s)", b="%1إ%3%4"},
+   {a="(a)(')(%p*)$", b="%1أ%3"},
+   {a="(a)(')(%p*%s)", b="%1أ%3"},
 -- i
-   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
-   {a="(i)(')(%p?)$", b="%1ئ%3"},
-   {a="(i)(')(%p?%s)", b="%1ئ%3"},
+   {a="(i)(')([uai]N?%p*)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p*%s)", b="%1ئ%3%4"},
+   {a="(i)(')(%p*)$", b="%1ئ%3"},
+   {a="(i)(')(%p*%s)", b="%1ئ%3"},
 --
    -- middle
    {a="([UIwy])(')", b="%1ء"}, --new
@@ -323,10 +330,10 @@ hamzaeasy = { -- differences marked below with 'easy'
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    --easy (begin)
-   {a="(A)(')(uN?%p?)$", b="aاء%3"},
-   {a="(A)(')(uN?)(%p?%s)", b="aاء%3%4"},
-   {a="(A)(')(iN?%p?)$", b="aاء%3"},
-   {a="(A)(')(iN?)(%p?%s)", b="aاء%3%4"},
+   {a="(A)(')(uN?%p*)$", b="aاء%3"},
+   {a="(A)(')(uN?)(%p*%s)", b="aاء%3%4"},
+   {a="(A)(')(iN?%p*)$", b="aاء%3"},
+   {a="(A)(')(iN?)(%p*%s)", b="aاء%3%4"},
    {a="(A)(')([iI])", b="aائ%3"}, -- historic madda
    {a="(A)(')(u)", b="aاؤ%3"}, -- historic madda
    {a="(A)(')", b="aاء"}, -- historic madda
@@ -360,38 +367,38 @@ hamzaeasy = { -- differences marked below with 'easy'
    -- hamzah alone on the line, so take out the following two lines
    -- (final hamzah is on the line after a letter of prolongation or a
    -- consonant with sukūn)
-   -- {a="([^Auai])(')(\"?[uai]N?)(%p?)$", b="%1ء%3%4"}, --new
-   -- {a="([^Auai])(')(\"?[uai]N?)(%p?%s)", b="%1ء%3%4"},
-   {a="([^Auai])(')(\"?aN)(%p?)$", b="%1ئ%3%4"}, --new
-   {a="([^Auai])(')(\"?aN)(%p?%s)", b="%1ئ%3%4"}, --new
-   {a="([^uai])(')(\"?a)(%p?)$", b="%1ء%3%4"}, --new
-   {a="([^uai])(')(\"?a)(%p?%s)", b="%1ء%3%4"}, --new
-   {a="([^uai])(')(\"?[ui]N?)(%p?)$", b="%1ء%3%4"}, --new
-   {a="([^uai])(')(\"?[ui]N?)(%p?%s)", b="%1ء%3%4"}, --new
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p*)$", b="%1ء%3%4"}, --new
+   -- {a="([^Auai])(')(\"?[uai]N?)(%p*%s)", b="%1ء%3%4"},
+   {a="([^Auai])(')(\"?aN)(%p*)$", b="%1ئ%3%4"}, --new
+   {a="([^Auai])(')(\"?aN)(%p*%s)", b="%1ئ%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p*)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?a)(%p*%s)", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p*)$", b="%1ء%3%4"}, --new
+   {a="([^uai])(')(\"?[ui]N?)(%p*%s)", b="%1ء%3%4"}, --new
    --easy (end)
 -- u
-   {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
-   {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
-   {a="(u)(')(%p?)$", b="%1ؤ%3"},
-   {a="(u)(')(%p?%s)", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?%p*)$", b="%1ؤ%3"},
+   {a="(u)(')([uai]N?)(%p*%s)", b="%1ؤ%3%4"},
+   {a="(u)(')(%p*)$", b="%1ؤ%3"},
+   {a="(u)(')(%p*%s)", b="%1ؤ%3"},
 -- a
-   {a="(a)(')(A%p?)$", b="%1آ"},
-   {a="(a)(')(A)(%p?%s)", b="%1آ%4"},
-   {a="(a)(')([u]N?%p?)$", b="%1أ%3"},
-   {a="(a)(')([u]N?)(%p?%s)", b="%1أ%3%4"},
-   {a="(a)(')(a%p?)$", b="%1أ%3"},
-   {a="(a)(')(a)(%p?%s)", b="%1أ%3%4"},
-   {a="(a)(')(aN%p?)$", b="%1أً"},
-   {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
-   {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
-   {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
-   {a="(a)(')(%p?)$", b="%1أ%3"},
-   {a="(a)(')(%p?%s)", b="%1أ%3"},
+   {a="(a)(')(A%p*)$", b="%1آ"},
+   {a="(a)(')(A)(%p*%s)", b="%1آ%4"},
+   {a="(a)(')([u]N?%p*)$", b="%1أ%3"},
+   {a="(a)(')([u]N?)(%p*%s)", b="%1أ%3%4"},
+   {a="(a)(')(a%p*)$", b="%1أ%3"},
+   {a="(a)(')(a)(%p*%s)", b="%1أ%3%4"},
+   {a="(a)(')(aN%p*)$", b="%1أً"},
+   {a="(a)(')(aN)(%p*%s)", b="%1أً%4"},
+   {a="(a)(')([i]N?%p*)$", b="%1إ%3"},
+   {a="(a)(')([i]N?)(%p*%s)", b="%1إ%3%4"},
+   {a="(a)(')(%p*)$", b="%1أ%3"},
+   {a="(a)(')(%p*%s)", b="%1أ%3"},
 -- i
-   {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
-   {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
-   {a="(i)(')(%p?)$", b="%1ئ%3"},
-   {a="(i)(')(%p?%s)", b="%1ئ%3"},
+   {a="(i)(')([uai]N?%p*)$", b="%1ئ%3"},
+   {a="(i)(')([uai]N?)(%p*%s)", b="%1ئ%3%4"},
+   {a="(i)(')(%p*)$", b="%1ئ%3"},
+   {a="(i)(')(%p*%s)", b="%1ئ%3"},
 --
    -- middle
    {a="([Uw])(')", b="%1ء"}, --new
@@ -567,12 +574,12 @@ trigraphs = { -- trigraphs or more
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ال"},
    -- the silent wāw
-   {a="uU(%p?)$", b="uو%1"},
-   {a="uU(%p?%s)", b="uو%1"},
-   {a="aU(%p?)$", b="aو%1"},
-   {a="aU(%p?%s)", b="aو%1"},
-   {a="iU(%p?)$", b="iو%1"},
-   {a="iU(%p?%s)", b="iو%1"},
+   {a="uU(%p*)$", b="uو%1"},
+   {a="uU(%p*%s)", b="uو%1"},
+   {a="aU(%p*)$", b="aو%1"},
+   {a="aU(%p*%s)", b="aو%1"},
+   {a="iU(%p*)$", b="iو%1"},
+   {a="iU(%p*%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
@@ -658,12 +665,12 @@ trigraphseasy = { -- differences marked below with 'easy'
    -- art. with waṣla + lunar consonant (i.e. what remains)
    {a="'l%-", b="ال"},
    -- the silent wāw
-   {a="uU(%p?)$", b="uو%1"},
-   {a="uU(%p?%s)", b="uو%1"},
-   {a="aU(%p?)$", b="aو%1"},
-   {a="aU(%p?%s)", b="aو%1"},
-   {a="iU(%p?)$", b="iو%1"},
-   {a="iU(%p?%s)", b="iو%1"},
+   {a="uU(%p*)$", b="uو%1"},
+   {a="uU(%p*%s)", b="uو%1"},
+   {a="aU(%p*)$", b="aو%1"},
+   {a="aU(%p*%s)", b="aو%1"},
+   {a="iU(%p*)$", b="iو%1"},
+   {a="iU(%p*%s)", b="iو%1"},
    -- words ending in -āT with silent wāw/yāʾ
    {a="(_a)UA", b="%1وا"},
    {a="(_a)U", b="%1و"},
@@ -672,16 +679,16 @@ trigraphseasy = { -- differences marked below with 'easy'
 
 digraphs = {
    -- ʾiʿrāb: straight double quote must be discarded
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},   
-   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?[uai])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[uai])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="%2%3"},   
+   {a="(%-)(\"?ayni)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?[uai])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[uai])(%p*)$", b="%2%3"},
    -- ʾiʿrāb (end)
    -- initial straight double quote gives a connective ʾalif
    {a="^\"[uai]", b="ٱ"},
